"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTailwindComponent = void 0;
var react_1 = __importDefault(require("react"));
var react_2 = require("react");
var classnames_1 = require("../utils/classnames");
var createTailwindComponent = function (tw, component, strings, tags) {
    var includesFunctionTags = tags.some(function (tag) { return typeof tag === "function"; });
    var preCompiledClassnames = !includesFunctionTags
        ? (0, classnames_1.compileClassNames)(strings.raw, tags, {})
        : "";
    var twComponent = react_1.default.forwardRef(function (props, ref) {
        var _a, _b;
        var _c = react_1.default.useState(props.focused || false), focused = _c[0], setFocused = _c[1];
        var classnames = (0, react_2.useMemo)(function () {
            var cn = includesFunctionTags
                ? (0, classnames_1.compileClassNames)(strings.raw, tags, props)
                : preCompiledClassnames;
            return cn
                .split(" ")
                .filter(function (c) { return (c.includes("focus") ? focused : true); })
                .map(function (c) { return c.replaceAll("focus:", "").replaceAll(":focus", ""); })
                .join(" ");
        }, [props, focused]);
        var _handleFocus = react_1.default.useCallback(function () {
            var _a;
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            setFocused(true);
            if ("onFocus" in props) {
                (_a = props).onFocus.apply(_a, args);
            }
        }, [props]);
        var _handleBlur = react_1.default.useCallback(function () {
            var _a;
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            setFocused(false);
            if ("onBlur" in props) {
                (_a = props).onBlur.apply(_a, args);
            }
        }, [props]);
        var isFocusable = "onBlur" in (component.propTypes || {}) &&
            "onFocus" in (component.propTypes || {});
        return react_1.default.createElement(component, __assign(__assign({}, props), { ref: ref, onFocus: isFocusable ? _handleFocus : (_a = props) === null || _a === void 0 ? void 0 : _a.onFocus, onBlur: isFocusable ? _handleBlur : (_b = props) === null || _b === void 0 ? void 0 : _b.onBlur, style: tw.style(classnames) }));
    });
    return twComponent;
};
exports.createTailwindComponent = createTailwindComponent;
